@@ -21,33 +21,38 @@ static int init_ffi_lib(PyObject *m)
 {
     PyObject *x;
     int i;
+    static char init_done = 0;
 
     if (PyType_Ready(&FFI_Type) < 0)
         return -1;
     if (PyType_Ready(&Lib_Type) < 0)
         return -1;
-    if (init_global_types_dict(FFI_Type.tp_dict) < 0)
-        return -1;
 
-    FFIError = PyErr_NewException("ffi.error", NULL, NULL);
-    if (FFIError == NULL)
-        return -1;
-    if (PyDict_SetItemString(FFI_Type.tp_dict, "error", FFIError) < 0)
-        return -1;
-    if (PyDict_SetItemString(FFI_Type.tp_dict, "CType",
-                             (PyObject *)&CTypeDescr_Type) < 0)
-        return -1;
-    if (PyDict_SetItemString(FFI_Type.tp_dict, "CData",
-                             (PyObject *)&CData_Type) < 0)
-        return -1;
-
-    for (i = 0; all_dlopen_flags[i].name != NULL; i++) {
-        x = PyInt_FromLong(all_dlopen_flags[i].value);
-        if (x == NULL || PyDict_SetItemString(FFI_Type.tp_dict,
-                                              all_dlopen_flags[i].name,
-                                              x) < 0)
+    if (!init_done) {
+        if (init_global_types_dict(FFI_Type.tp_dict) < 0)
             return -1;
-        Py_DECREF(x);
+
+        FFIError = PyErr_NewException("ffi.error", NULL, NULL);
+        if (FFIError == NULL)
+            return -1;
+        if (PyDict_SetItemString(FFI_Type.tp_dict, "error", FFIError) < 0)
+            return -1;
+        if (PyDict_SetItemString(FFI_Type.tp_dict, "CType",
+                                 (PyObject *)&CTypeDescr_Type) < 0)
+            return -1;
+        if (PyDict_SetItemString(FFI_Type.tp_dict, "CData",
+                                 (PyObject *)&CData_Type) < 0)
+            return -1;
+
+        for (i = 0; all_dlopen_flags[i].name != NULL; i++) {
+            x = PyInt_FromLong(all_dlopen_flags[i].value);
+            if (x == NULL || PyDict_SetItemString(FFI_Type.tp_dict,
+                                                  all_dlopen_flags[i].name,
+                                                  x) < 0)
+                return -1;
+            Py_DECREF(x);
+        }
+        init_done = 1;
     }
 
     x = (PyObject *)&FFI_Type;
