@@ -13,6 +13,7 @@
 
 struct CPyExtFunc_s {
     PyMethodDef md;
+    void *direct_fn;
     int type_index;
 };
 static const char cpyextfunc_doc[] =
@@ -43,20 +44,9 @@ static struct CPyExtFunc_s *_cpyextfunc_get(PyObject *x)
     return exf;
 }
 
-static PyObject *_cpyextfunc_type_index(PyObject *x)
+static PyObject *_cpyextfunc_type(LibObject *lib, struct CPyExtFunc_s *exf)
 {
-    struct CPyExtFunc_s *exf;
-    LibObject *lib;
     PyObject *tuple, *result;
-
-    assert(PyErr_Occurred());
-    exf = _cpyextfunc_get(x);
-    if (exf == NULL)
-        return NULL;    /* still the same exception is set */
-
-    PyErr_Clear();
-
-    lib = (LibObject *)PyCFunction_GET_SELF(x);
     tuple = realize_c_type_or_func(lib->l_types_builder,
                                    lib->l_types_builder->ctx.types,
                                    exf->type_index);
@@ -71,6 +61,22 @@ static PyObject *_cpyextfunc_type_index(PyObject *x)
     return result;
 }
 
+static PyObject *_cpyextfunc_type_index(PyObject *x)
+{
+    struct CPyExtFunc_s *exf;
+    LibObject *lib;
+
+    assert(PyErr_Occurred());
+    exf = _cpyextfunc_get(x);
+    if (exf == NULL)
+        return NULL;    /* still the same exception is set */
+
+    PyErr_Clear();
+
+    lib = (LibObject *)PyCFunction_GET_SELF(x);
+    return _cpyextfunc_type(lib, exf);
+}
+
 static void cdlopen_close_ignore_errors(void *libhandle);  /* forward */
 static void *cdlopen_fetch(PyObject *libname, void *libhandle, char *symbol);
 
@@ -109,23 +115,30 @@ static PyObject *lib_build_cpython_func(LibObject *lib,
     struct CPyExtFunc_s *xfunc;
     int i, type_index = _CFFI_GETARG(g->type_op);
     _cffi_opcode_t *opcodes = lib->l_types_builder->ctx.types;
-    assert(_CFFI_GETOP(opcodes[type_index]) == _CFFI_OP_FUNCTION);
 
-    /* return type: */
-    ct = realize_c_type(lib->l_types_builder, opcodes,
-                        _CFFI_GETARG(opcodes[type_index]));
-    if (ct == NULL)
-        return NULL;
-    Py_DECREF(ct);
+    if ((((uintptr_t)opcodes[type_index]) & 1) == 0) {
+        /* the function type was already built.  No need to force
+           the arg and return value to be built again. */
+    }
+    else {
+        assert(_CFFI_GETOP(opcodes[type_index]) == _CFFI_OP_FUNCTION);
 
-    /* argument types: */
-    i = type_index + 1;
-    while (_CFFI_GETOP(opcodes[i]) != _CFFI_OP_FUNCTION_END) {
-        ct = realize_c_type(lib->l_types_builder, opcodes, i);
+        /* return type: */
+        ct = realize_c_type(lib->l_types_builder, opcodes,
+                            _CFFI_GETARG(opcodes[type_index]));
         if (ct == NULL)
             return NULL;
         Py_DECREF(ct);
-        i++;
+
+        /* argument types: */
+        i = type_index + 1;
+        while (_CFFI_GETOP(opcodes[i]) != _CFFI_OP_FUNCTION_END) {
+            ct = realize_c_type(lib->l_types_builder, opcodes, i);
+            if (ct == NULL)
+                return NULL;
+            Py_DECREF(ct);
+            i++;
+        }
     }
 
     /* xxx the few bytes of memory we allocate here leak, but it's a
@@ -144,6 +157,7 @@ static PyObject *lib_build_cpython_func(LibObject *lib,
     xfunc->md.ml_flags = flags;
     xfunc->md.ml_name = g->name;
     xfunc->md.ml_doc = cpyextfunc_doc;
+    xfunc->direct_fn = g->size_or_direct_fn;
     xfunc->type_index = type_index;
 
     return PyCFunction_NewEx(&xfunc->md, (PyObject *)lib, lib->l_libname);
@@ -243,6 +257,7 @@ static PyObject *lib_build_and_cache_attr(LibObject *lib, PyObject *name,
     }
 
     case _CFFI_OP_CONSTANT:
+    case _CFFI_OP_DLOPEN_CONST:
     {
         /* a constant which is not of integer type */
         char *data;
@@ -251,26 +266,54 @@ static PyObject *lib_build_and_cache_attr(LibObject *lib, PyObject *name,
         if (ct == NULL)
             return NULL;
 
-        assert(g->address);
-        assert(ct->ct_size > 0);
-        data = alloca(ct->ct_size);
-        ((void(*)(char*))g->address)(data);
+        if (ct->ct_size <= 0) {
+            PyErr_Format(FFIError, "constant '%s' is of type '%s', "
+                         "whose size is not known", s, ct->ct_name);
+            return NULL;
+        }
+        if (g->address == NULL) {
+            /* for dlopen() style */
+            assert(_CFFI_GETOP(g->type_op) == _CFFI_OP_DLOPEN_CONST);
+            data = cdlopen_fetch(lib->l_libname, lib->l_libhandle, s);
+            if (data == NULL)
+                return NULL;
+        }
+        else {
+            /* xxx the few bytes of memory we allocate here leak, but it's
+               a minor concern because it should only occur for
+               OP_CONSTANT.  There is one per real non-integer C constant
+               in a CFFI C extension module.  CPython never unloads its C
+               extension modules anyway.  Note that we used to do alloca(),
+               but see issue #198. */
+            assert(_CFFI_GETOP(g->type_op) == _CFFI_OP_CONSTANT);
+            data = PyMem_Malloc(ct->ct_size);
+            if (data == NULL) {
+                PyErr_NoMemory();
+                return NULL;
+            }
+            ((void(*)(char*))g->address)(data);
+        }
         x = convert_to_object(data, ct);
         Py_DECREF(ct);
         break;
     }
 
     case _CFFI_OP_GLOBAL_VAR:
-        /* global variable of the exact type specified here */
+    {
+        /* global variable of the exact type specified here
+           (nowadays, only used by the ABI mode or backward
+           compatibility; see _CFFI_OP_GLOBAL_VAR_F for the API mode)
+         */
+        Py_ssize_t g_size = (Py_ssize_t)g->size_or_direct_fn;
         ct = realize_c_type(types_builder, types_builder->ctx.types,
                             _CFFI_GETARG(g->type_op));
         if (ct == NULL)
             return NULL;
-        if (g->size != ct->ct_size && g->size != 0 && ct->ct_size > 0) {
+        if (g_size != ct->ct_size && g_size != 0 && ct->ct_size > 0) {
             PyErr_Format(FFIError,
                          "global variable '%.200s' should be %zd bytes "
                          "according to the cdef, but is actually %zd",
-                         s, ct->ct_size, g->size);
+                         s, ct->ct_size, g_size);
             x = NULL;
         }
         else {
@@ -281,10 +324,20 @@ static PyObject *lib_build_and_cache_attr(LibObject *lib, PyObject *name,
                 if (address == NULL)
                     return NULL;
             }
-            x = make_global_var(ct, address);
+            x = make_global_var(name, ct, address, NULL);
         }
         Py_DECREF(ct);
         break;
+    }
+
+    case _CFFI_OP_GLOBAL_VAR_F:
+        ct = realize_c_type(types_builder, types_builder->ctx.types,
+                            _CFFI_GETARG(g->type_op));
+        if (ct == NULL)
+            return NULL;
+        x = make_global_var(name, ct, NULL, (gs_fetch_addr_fn)g->address);
+        Py_DECREF(ct);
+        break;
 
     case _CFFI_OP_DLOPEN_FUNC:
     {
@@ -338,16 +391,88 @@ static PyObject *lib_build_and_cache_attr(LibObject *lib, PyObject *name,
         }                                               \
     } while (0)
 
+static PyObject *_lib_dir1(LibObject *lib, int ignore_global_vars)
+{
+    const struct _cffi_global_s *g = lib->l_types_builder->ctx.globals;
+    int i, count = 0, total = lib->l_types_builder->ctx.num_globals;
+    PyObject *s, *lst = PyList_New(total);
+    if (lst == NULL)
+        return NULL;
+
+    for (i = 0; i < total; i++) {
+        if (ignore_global_vars) {
+            int op = _CFFI_GETOP(g[i].type_op);
+            if (op == _CFFI_OP_GLOBAL_VAR || op == _CFFI_OP_GLOBAL_VAR_F)
+                continue;
+        }
+        s = PyText_FromString(g[i].name);
+        if (s == NULL)
+            goto error;
+        PyList_SET_ITEM(lst, count, s);
+        count++;
+    }
+    if (PyList_SetSlice(lst, count, total, NULL) < 0)
+        goto error;
+    return lst;
+
+ error:
+    Py_DECREF(lst);
+    return NULL;
+}
+
+static PyObject *_lib_dict(LibObject *lib)
+{
+    const struct _cffi_global_s *g = lib->l_types_builder->ctx.globals;
+    int i, total = lib->l_types_builder->ctx.num_globals;
+    PyObject *name, *x, *d = PyDict_New();
+    if (d == NULL)
+        return NULL;
+
+    for (i = 0; i < total; i++) {
+        name = PyText_FromString(g[i].name);
+        if (name == NULL)
+            goto error;
+
+        LIB_GET_OR_CACHE_ADDR(x, lib, name, goto error);
+
+        if (PyDict_SetItem(d, name, x) < 0)
+            goto error;
+        Py_DECREF(name);
+    }
+    return d;
+
+ error:
+    Py_XDECREF(name);
+    Py_DECREF(d);
+    return NULL;
+}
+
 static PyObject *lib_getattr(LibObject *lib, PyObject *name)
 {
     PyObject *x;
-    LIB_GET_OR_CACHE_ADDR(x, lib, name, return NULL);
+    LIB_GET_OR_CACHE_ADDR(x, lib, name, goto missing);
 
     if (GlobSupport_Check(x)) {
         return read_global_var((GlobSupportObject *)x);
     }
     Py_INCREF(x);
     return x;
+
+ missing:
+    if (strcmp(PyText_AsUTF8(name), "__all__") == 0) {
+        PyErr_Clear();
+        return _lib_dir1(lib, 1);
+    }
+    if (strcmp(PyText_AsUTF8(name), "__dict__") == 0) {
+        PyErr_Clear();
+        return _lib_dict(lib);
+    }
+    /* this hack is for Python 3.5 */
+    if (strcmp(PyText_AsUTF8(name), "__name__") == 0) {
+        PyErr_Clear();
+        return lib_repr(lib);
+    }
+    return NULL;
 }
 
 static int lib_setattr(LibObject *lib, PyObject *name, PyObject *val)
@@ -370,27 +495,13 @@ static int lib_setattr(LibObject *lib, PyObject *name, PyObject *val)
     return -1;
 }
 
-static PyObject *lib_dir(LibObject *lib, PyObject *noarg)
+static PyObject *lib_dir(PyObject *self, PyObject *noarg)
 {
-    const struct _cffi_global_s *g = lib->l_types_builder->ctx.globals;
-    int i, total = lib->l_types_builder->ctx.num_globals;
-    PyObject *lst = PyList_New(total);
-    if (lst == NULL)
-        return NULL;
-
-    for (i = 0; i < total; i++) {
-        PyObject *s = PyText_FromString(g[i].name);
-        if (s == NULL) {
-            Py_DECREF(lst);
-            return NULL;
-        }
-        PyList_SET_ITEM(lst, i, s);
-    }
-    return lst;
+    return _lib_dir1((LibObject *)self, 0);
 }
 
 static PyMethodDef lib_methods[] = {
-    {"__dir__",   (PyCFunction)lib_dir,  METH_NOARGS},
+    {"__dir__",   lib_dir,  METH_NOARGS},
     {NULL,        NULL}           /* sentinel */
 };
 
@@ -489,9 +600,21 @@ static PyObject *address_of_global_var(PyObject *args)
     }
     else {
         struct CPyExtFunc_s *exf = _cpyextfunc_get(x);
-        if (exf != NULL  ||  /* an OP_CPYTHON_BLTN: '&func' is 'func' in C */
-            ((CData_Check(x) &&  /* or, a constant functionptr cdata: same */
-              (((CDataObject *)x)->c_type->ct_flags & CT_FUNCTIONPTR) != 0))) {
+        if (exf != NULL) {  /* an OP_CPYTHON_BLTN: '&func' returns a cdata */
+            PyObject *ct;
+            if (exf->direct_fn == NULL) {
+                Py_INCREF(x);    /* backward compatibility */
+                return x;
+            }
+            ct = _cpyextfunc_type(lib, exf);
+            if (ct == NULL)
+                return NULL;
+            x = new_simple_cdata(exf->direct_fn, (CTypeDescrObject *)ct);
+            Py_DECREF(ct);
+            return x;
+        }
+        if (CData_Check(x) &&  /* a constant functionptr cdata: 'f == &f' */
+                (((CDataObject *)x)->c_type->ct_flags & CT_FUNCTIONPTR) != 0) {
             Py_INCREF(x);
             return x;
         }
