@@ -14,9 +14,10 @@ static CTypeDescrObject *g_ct_voidp, *g_ct_chararray;
 
 static PyObject *build_primitive_type(int num);   /* forward */
 
+#define primitive_in_range(num)   ((num) >= 0 && (num) < _CFFI__NUM_PRIM)
 #define get_primitive_type(num)                                 \
-    (all_primitives[num] != NULL ? all_primitives[num]          \
-                                 : build_primitive_type(num))
+    ((primitive_in_range(num) && all_primitives[num] != NULL) ? \
+        all_primitives[num] : build_primitive_type(num))
 
 static int init_global_types_dict(PyObject *ffi_type_dict)
 {
@@ -59,7 +60,7 @@ static int init_global_types_dict(PyObject *ffi_type_dict)
 static void free_builder_c(builder_c_t *builder, int ctx_is_static)
 {
     if (!ctx_is_static) {
-        int i;
+        size_t i;
         const void *mem[] = {builder->ctx.types,
                              builder->ctx.globals,
                              builder->ctx.struct_unions,
@@ -153,14 +154,29 @@ static PyObject *build_primitive_type(int num)
     };
     PyObject *x;
 
+    assert(sizeof(primitive_name) == sizeof(*primitive_name) * _CFFI__NUM_PRIM);
     if (num == _CFFI_PRIM_VOID) {
         x = new_void_type();
     }
-    else if (0 <= num &&
-             num < sizeof(primitive_name) / sizeof(*primitive_name) &&
-             primitive_name[num] != NULL) {
+    else if (primitive_in_range(num) && primitive_name[num] != NULL) {
         x = new_primitive_type(primitive_name[num]);
     }
+    else if (num == _CFFI__UNKNOWN_PRIM) {
+        PyErr_SetString(FFIError, "primitive integer type with an unexpected "
+                        "size (or not an integer type at all)");
+        return NULL;
+    }
+    else if (num == _CFFI__UNKNOWN_FLOAT_PRIM) {
+        PyErr_SetString(FFIError, "primitive floating-point type with an "
+                        "unexpected size (or not a float type at all)");
+        return NULL;
+    }
+    else if (num == _CFFI__UNKNOWN_LONG_DOUBLE) {
+        PyErr_SetString(FFIError, "primitive floating-point type is "
+                        "'long double', not supported for now with "
+                        "the syntax 'typedef double... xxx;'");
+        return NULL;
+    }
     else {
         PyErr_Format(PyExc_NotImplementedError, "prim=%d", num);
         return NULL;
@@ -528,7 +544,7 @@ realize_c_type_or_func(builder_c_t *builder,
     case _CFFI_OP_FUNCTION:
     {
         PyObject *fargs;
-        int i, base_index, num_args, ellipsis;
+        int i, base_index, num_args, ellipsis, abi;
 
         y = (PyObject *)realize_c_type(builder, opcodes, _CFFI_GETARG(op));
         if (y == NULL)
@@ -536,11 +552,32 @@ realize_c_type_or_func(builder_c_t *builder,
 
         base_index = index + 1;
         num_args = 0;
+        /* note that if the arguments are already built, they have a
+           pointer in the 'opcodes' array, and GETOP() returns a
+           random even value.  But OP_FUNCTION_END is odd, so the
+           condition below still works correctly. */
         while (_CFFI_GETOP(opcodes[base_index + num_args]) !=
                    _CFFI_OP_FUNCTION_END)
             num_args++;
 
-        ellipsis = _CFFI_GETARG(opcodes[base_index + num_args]) & 1;
+        ellipsis = _CFFI_GETARG(opcodes[base_index + num_args]) & 0x01;
+        abi      = _CFFI_GETARG(opcodes[base_index + num_args]) & 0xFE;
+        switch (abi) {
+        case 0:
+            abi = FFI_DEFAULT_ABI;
+            break;
+        case 2:
+#if defined(MS_WIN32) && !defined(_WIN64)
+            abi = FFI_STDCALL;
+#else
+            abi = FFI_DEFAULT_ABI;
+#endif
+            break;
+        default:
+            PyErr_Format(FFIError, "abi number %d not supported", abi);
+            Py_DECREF(y);
+            return NULL;
+        }
 
         fargs = PyTuple_New(num_args);
         if (fargs == NULL) {
@@ -558,8 +595,7 @@ realize_c_type_or_func(builder_c_t *builder,
             PyTuple_SET_ITEM(fargs, i, z);
         }
 
-        z = new_function_type(fargs, (CTypeDescrObject *)y, ellipsis,
-                              FFI_DEFAULT_ABI);
+        z = new_function_type(fargs, (CTypeDescrObject *)y, ellipsis, abi);
         Py_DECREF(fargs);
         Py_DECREF(y);
         if (z == NULL)
